//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*----------------------------------------------------------------------------
//* File Name           : main.c
//* Object              : main application written in C
//* Creation            : JPP   16/Jun/2004
//*----------------------------------------------------------------------------


//*----------------------------------------------------------------------------
//*        Program modified by John for Olimex AT91SAM7-P64 board
//*  Modification on main.c for led_mask[] and a conditional statement in change_speed()
//*
//*  1. Comment the original const int led_mask[8]= {LED1, LED2, LED3, LED4}
//*  2. #define const int led_mask[2]={LED1,LED2};
//*  3. define switch for SW1 and SW2 only (under change_speed())
//*  4. Replace the original code AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_PIOA )
//*       by  *AT91C_PMC_SCER = AT91C_CKGR_MOSCEN .....
//*  5. Add the line     //*AT91C_RSTC_RMR = (0xA5000000|AT91C_RSTC_URSTEN); for NRST detection
//*
//*  Date       : 31 Jan 2006
//*  Programmer : John Leung, TechToys, www.TechToys.com.hk
//*----------------------------------------------------------------------------


// Include Standard files
#include "Board.h"

/* Global variables */
#define SPEED 		(MCKKHz/10)
unsigned int LedSpeed = SPEED *50 ;
//const int led_mask[8]= {LED1, LED2, LED3, LED4};
const int led_mask[2]={LED1,LED2};

//*--------------------------------------------------------------------------------------
//* Function Name       : change_speed
//* Object              : Adjust "LedSpeed" value depending on SW1 and SW2 are pressed or not
//* Input Parameters    : none
//* Output Parameters   : Update of LedSpeed value.
//*--------------------------------------------------------------------------------------
void change_speed ( void )
{//* Begin
    if ( (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & SW1_MASK) == 0 )
    {
        if ( LedSpeed > SPEED ) LedSpeed -=SPEED ;
    }

    if ( (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & SW2_MASK) == 0 )
    {
        if ( LedSpeed < MCK ) LedSpeed +=SPEED ;
    }
}//* End

//*--------------------------------------------------------------------------------------
//* Function Name       : wait
//* Object              : Software waiting loop
//* Input Parameters    : none. Waiting time is defined by the global variable LedSpeed.
//* Output Parameters   : none
//*--------------------------------------------------------------------------------------
void wait ( void )
{//* Begin
    unsigned int waiting_time ;
    change_speed () ;
    for(waiting_time = 0; waiting_time < LedSpeed; waiting_time++) ;
}//* End

//*--------------------------------------------------------------------------------------
//* Function Name       : Main
//* Object              : Software entry point
//* Input Parameters    : none.
//* Output Parameters   : none.
//*--------------------------------------------------------------------------------------
int main()
{//* Begin
    int i;

    //*AT91C_RSTC_RMR = (0xA5000000|AT91C_RSTC_URSTEN);

    // First, enable the clock of the PIO because peripheral clocks disabled after reset
    // Peripheral Clock required for SW1 & SW2 only. According to SAM7S64 manual (page 227),
    // the Power Management Controller (PMC) controls the PIO Controller clock in order to save power.
    // Writing any of the registers of the user interface does not require the PIO Controller clock to be
    // enabled. This means that the configuration of the I/O lines does not require the PIO Controller
    // clock to be enabled. Thus, output LED1 or LED2 high / low doesn't require PMC.
    // However, when the clock is disabled, not all of the features of the PIO Controller are available.
    // Note that the Input Change Interrupt and the read of the pin level require the clock to be
    // validated. Therefore, we need enabling the PMC only for input lines at SW1 & SW2


       //AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_PIOA ) ;
       *AT91C_PMC_SCER = AT91C_CKGR_MOSCEN;   // main oscillator enable
       *AT91C_PMC_PCER = 1<<AT91C_ID_PIOA;    // peripheral clock enable for SW1 and SW2 only

    // then, we configure the PIO Lines corresponding to LED1 to LED2
    // to be outputs. No need to set these pins to be driven by the PIO because it is GPIO pins only.
      //AT91F_PIO_CfgOutput( AT91C_BASE_PIOA, LED_MASK ) ;
       *AT91C_PIOA_PER = LED_MASK;
       *AT91C_PIOA_OER = LED_MASK;

    //  AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED_MASK ) ;
    //  Clear the LED's. On the SAM7S-P64 board we must apply a "1" to turn off LEDs
       *AT91C_PIOA_SODR = LED_MASK;

    // Loop forever
    for (;;)
    {
        // Once a Shot on each led
	for ( i=0 ; i < NB_LEB ; i++ )
        {
            *AT91C_PIOA_CODR = led_mask[i];  //clear output data register, turn ON LED
	    wait();
            *AT91C_PIOA_SODR = led_mask[i];  //set output data register, turn OFF LED
	    wait();
        }// End for
    }// End for
}//* End
